==> python-utmp_0.2.tar.gz: python-utmp-0.2/utmpaccessmodule.c <==
#include "Python.h"

#include <utmp.h>

#include "constants.h"

/* define out "ideal" struct utmp, based on glibc utmp */
struct myutmp {
    short ut_type;      		/* type of login */
    pid_t ut_pid;	        	/* pid of login process */
    char ut_line[UT_LINESIZE + 1];	/* device name of tty - "/dev/" */
    char ut_id[4 + 1];		        /* init id or abbrev. ttyname */
    char ut_user[UT_NAMESIZE + 1];	/* user name */
    char ut_host[UT_HOSTSIZE + 1];	/* hostname for remote login */
    short int e_termination;    	/* process termination status.  */
    short int e_exit;		        /* process exit status.  */
    long ut_session;		        /* session ID, used for windowing */
    long tv_sec;		        /* time entry was made.  */
    long tv_usec;
    int32_t ut_addr_v6[4];	        /* IP address of remote host.  */
};


/* copy system utmp to our ideal one, using default values
   if system utmp does not have all the fields we have
*/
void system2my(m, s)
struct myutmp *m;
struct utmp *s;
{
#ifdef _HAVE_UT_TYPE
    m->ut_type = s->ut_type;
#else
    m->ut_type = USER_PROCESS; /* pretend everything to be of type USER_PROCESS */
#endif                         /* if system utmp does not know about type */

#ifdef _HAVE_UT_PID
    m->ut_pid = s->ut_pid;
#else
    m->ut_pid = 0;
#endif

    strncpy(m->ut_line, s->ut_line, UT_LINESIZE);

#ifdef _HAVE_UT_ID
    strncpy(m->ut_id, s->ut_id, 4);
#else
    m->ut_id[0] = '\0';
#endif

    strncpy(m->ut_user, s->ut_user, UT_NAMESIZE);

#ifdef _HAVE_UT_HOST
    strncpy(m->ut_host, s->ut_host, UT_HOSTSIZE);
#else
    m->ut_host[0] = '\0';
#endif

#ifdef _HAVE_UT_EXIT
    m->e_termination = s->ut_exit.e_termination;
    m->e_exit = s->ut_exit.e_exit;
#else
    m->e_termination = 0;
    m->e_exit = 0;
#endif

#ifdef _HAVE_UT_TV
    m->tv_sec = s->ut_tv.tv_sec;
    m->tv_usec = s->ut_tv.tv_usec;
#else
    m->tv_sec = s->ut_time;
    m->tv_usec = 0;
#endif

#ifdef _HAVE_UT_ADDR_V6
    memcpy(m->ut_addr_v6, s->ut_addr_v6, 4 * sizeof(int32_t));
#else
    memset(m->ut_addr_v6, 0, 4 * sizeof(int32_t));
#endif

#ifdef _HAVE_UT_SESSION
    m->ut_session = s->ut_session;
#else
    m->ut_session = 0;
#endif
    m->ut_line[UT_LINESIZE] = '\0';
    m->ut_user[UT_NAMESIZE] = '\0';
    m->ut_host[UT_HOSTSIZE] = '\0';
    m->ut_id[4] = '\0';
}


/* copy our ideal utmp to system one, copy only
   those fields system utmp has
*/
void my2system(s, m)
struct myutmp *m;
struct utmp *s;
{
#ifdef _HAVE_UT_TYPE
    s->ut_type = m->ut_type;
#endif

#ifdef _HAVE_UT_PID
    s->ut_pid = m->ut_pid;
#endif

    strncpy(s->ut_line, m->ut_line, UT_LINESIZE);

#ifdef _HAVE_UT_ID
    strncpy(s->ut_id, m->ut_id, 4);
#endif

    strncpy(s->ut_user, m->ut_user, UT_NAMESIZE);

#ifdef _HAVE_UT_HOST
    strncpy(s->ut_host, m->ut_host, UT_HOSTSIZE);
#endif

#ifdef _HAVE_UT_EXIT
    s->ut_exit.e_termination = m->e_termination;
    s->ut_exit.e_exit = m->e_exit;
#endif

#ifdef _HAVE_UT_TV
    s->ut_tv.tv_sec = m->tv_sec;
    s->ut_tv.tv_usec = m->tv_usec;
#else
    s->ut_time = m->tv_sec;
#endif

#ifdef _HAVE_UT_ADDR_V6
    memcpy(s->ut_addr_v6, m->ut_addr_v6, 4 * sizeof(int32_t));
#endif

#ifdef _HAVE_UT_SESSION
    s->ut_session = m->ut_session;
#endif
}

/* #define OUTVALUE(entry) \
                "(iissssiiiiii)", \
                entry->ut_type, \
                entry->ut_pid, \
                entry->ut_line, \
                entry->ut_id, \
                entry->ut_user, \
                entry->ut_host, \
                entry->ut_exit.e_termination, \
                entry->ut_exit.e_exit, \
                entry->ut_session, \
                entry->ut_tv.tv_sec, \
                entry->ut_tv.tv_usec, \
                entry->ut_addr_v6
*/

#define OUTVALUE(entry) \
                "(iissssiiiiii)", \
                entry->ut_type, \
                entry->ut_pid, \
                entry->ut_line, \
                entry->ut_id, \
                entry->ut_user, \
                entry->ut_host, \
                entry->e_termination, \
                entry->e_exit, \
                entry->ut_session, \
                entry->tv_sec, \
                entry->tv_usec, \
                entry->ut_addr_v6


static PyObject *utmp_utmpname(self, args)
PyObject *self;
PyObject *args;
{
    char *fname;

    if (!PyArg_ParseTuple(args, "s", &fname))
	return NULL;
    utmpname(fname);
    Py_INCREF(Py_None);
    return Py_None;
}


static PyObject *utmp_setutent(self, args)
PyObject *self;
PyObject *args;
{
    if (!PyArg_ParseTuple(args, ""))
	return NULL;
    setutent();
    Py_INCREF(Py_None);
    return Py_None;
}



static PyObject *utmp_getutent(self, args)
PyObject *self;
PyObject *args;
{
    struct utmp *entry;
    struct myutmp myentry;

    if (!PyArg_ParseTuple(args, ""))
	return NULL;
    entry = getutent();

    if (!entry) {
	Py_INCREF(Py_None);
	return Py_None;
    };
    system2my(&myentry, entry);

    return Py_BuildValue(OUTVALUE((&myentry)));
}


static PyObject *utmp_getutid(self, args) /* args are ut_type=...[, ut_id=...] */
PyObject *self;
PyObject *args;
{
    struct myutmp myentry;
    struct utmp entry;
    struct utmp *nentry;
    struct myutmp mynentry;
    short ut_type;
    char *ut_id = "";


    if (!PyArg_ParseTuple(args, "h|s", &ut_type, &ut_id))
	return NULL;

    myentry.ut_type = ut_type;
    strncpy(myentry.ut_id, ut_id, 4);
    my2system(&entry, &myentry);
    nentry = getutid(&entry);

    if (!nentry) {
	Py_INCREF(Py_None);
	return Py_None;
    };

    system2my(&mynentry, nentry);
    return Py_BuildValue(OUTVALUE((&mynentry)));
}


static PyObject *utmp_getutline(self, args) /* args is ut_line */
PyObject *self;
PyObject *args;
{
    struct utmp entry;
    struct utmp *nentry;
    struct myutmp mynentry;
    char *ut_line;


    if (!PyArg_ParseTuple(args, "s", &ut_line))
	return NULL;

    strncpy(entry.ut_line, ut_line, UT_LINESIZE);
    nentry = getutline(&entry);

    if (!nentry) {
	Py_INCREF(Py_None);
	return Py_None;
    };

    system2my(&mynentry, nentry);
    return Py_BuildValue(OUTVALUE((&mynentry)));
}


static PyObject *utmp_pututline(self, args)
PyObject *self;
PyObject *args;
{
    struct utmp entry;
    struct myutmp myentry;
    char *ut_line, *ut_id, *ut_user, *ut_host;

    if (!PyArg_ParseTuple(args, "iissssiiiiii",
			  &myentry.ut_type, &myentry.ut_pid, &ut_line,
			  &ut_id, &ut_user, &ut_host,
			  &myentry.e_termination, &myentry.e_exit,
			  &myentry.ut_session, &myentry.tv_sec,
			  &myentry.tv_usec, &myentry.ut_addr_v6))
	return NULL;

    strncpy(myentry.ut_line, ut_line, UT_LINESIZE);
    strncpy(myentry.ut_id, ut_id, 4);
    strncpy(myentry.ut_user, ut_user, UT_NAMESIZE);
    strncpy(myentry.ut_host, ut_host, UT_HOSTSIZE);

    my2system(&entry, &myentry);
#ifdef _HAVE_UT_TYPE
    pututline(&entry);
#else
    if (myentry.ut_type != USER_PROCESS)
        memset(&entry, 0, sizeof(entry));
    pututline(&entry);
#endif
    Py_INCREF(Py_None);
    return Py_None;
}



static PyObject *utmp_endutent(self, args)
PyObject *self;
PyObject *args;
{
    if (!PyArg_ParseTuple(args, ""))
	return NULL;
    endutent();
    Py_INCREF(Py_None);
    return Py_None;
}


static PyMethodDef UtmpMethods[] = {
    {"utmpname", utmp_utmpname, METH_VARARGS},
    {"setutent", utmp_setutent, METH_VARARGS},
    {"getutent", utmp_getutent, METH_VARARGS},
    {"endutent", utmp_endutent, METH_VARARGS},
    {"getutid", utmp_getutid, METH_VARARGS},
    {"getutline", utmp_getutline, METH_VARARGS},
    {"pututline", utmp_pututline, METH_VARARGS},
    {NULL, NULL}		/* Sentinel */
};


void initutmpaccess()
{
    (void) Py_InitModule("utmpaccess", UtmpMethods);
}

==> python-utmp_0.3.tar.gz: python-utmp-0.3/utmpaccessmodule.c <==
#include "Python.h"
#include <utmp.h>

#include "constants.h"

/* define out "ideal" struct utmp, based on glibc utmp */
struct myutmp {
    short ut_type;		/* type of login */
    pid_t ut_pid;		/* pid of login process */
    char ut_line[UT_LINESIZE + 1];	/* device name of tty - "/dev/" */
    char ut_id[4 + 1];		/* init id or abbrev. ttyname */
    char ut_user[UT_NAMESIZE + 1];	/* user name */
    char ut_host[UT_HOSTSIZE + 1];	/* hostname for remote login */
    short int e_termination;	/* process termination status.  */
    short int e_exit;		/* process exit status.  */
    long ut_session;		/* session ID, used for windowing */
    long tv_sec;		/* time entry was made.  */
    long tv_usec;
    int32_t ut_addr_v6[4];	/* IP address of remote host.  */
};


/* copy system utmp to our ideal one, using default values
   if system utmp does not have all the fields we have
*/
void system2my(m, s)
struct myutmp *m;
struct utmp *s;
{
#ifdef _HAVE_UT_TYPE
    m->ut_type = s->ut_type;
#else
    m->ut_type = USER_PROCESS;	/* pretend everything to be of type USER_PROCESS */
    if ((s->ut_user[0] == '\0') && (s->ut_host[0] == '\0')) {
	if (s->ut_line[0] == '\0')	/* empty record */
	    m->ut_type = EMPTY;
	else
	    m->ut_type = DEAD_PROCESS;
    }
#endif

#ifdef _HAVE_UT_PID
    m->ut_pid = s->ut_pid;
#else
    m->ut_pid = 0;
#endif

    strncpy(m->ut_line, s->ut_line, UT_LINESIZE);

#ifdef _HAVE_UT_ID
    strncpy(m->ut_id, s->ut_id, 4);
#else
    m->ut_id[0] = '\0';
#endif

    strncpy(m->ut_user, s->ut_user, UT_NAMESIZE);

#ifdef _HAVE_UT_HOST
    strncpy(m->ut_host, s->ut_host, UT_HOSTSIZE);
#else
    m->ut_host[0] = '\0';
#endif

#ifdef _HAVE_UT_EXIT
    m->e_termination = s->ut_exit.e_termination;
    m->e_exit = s->ut_exit.e_exit;
#else
    m->e_termination = 0;
    m->e_exit = 0;
#endif

#ifdef _HAVE_UT_TV
    m->tv_sec = s->ut_tv.tv_sec;
    m->tv_usec = s->ut_tv.tv_usec;
#else
    m->tv_sec = s->ut_time;
    m->tv_usec = 0;
#endif

#ifdef _HAVE_UT_ADDR_V6
    memcpy(m->ut_addr_v6, s->ut_addr_v6, 4 * sizeof(int32_t));
#else
    memset(m->ut_addr_v6, 0, 4 * sizeof(int32_t));
#endif

#ifdef _HAVE_UT_SESSION
    m->ut_session = s->ut_session;
#else
    m->ut_session = 0;
#endif
    m->ut_line[UT_LINESIZE] = '\0';
    m->ut_user[UT_NAMESIZE] = '\0';
    m->ut_host[UT_HOSTSIZE] = '\0';
    m->ut_id[4] = '\0';
}


/* copy our ideal utmp to system one, copy only
   those fields system utmp has
*/
void my2system(s, m)
struct myutmp *m;
struct utmp *s;
{
#ifdef _HAVE_UT_TYPE
    s->ut_type = m->ut_type;
#endif

#ifdef _HAVE_UT_PID
    s->ut_pid = m->ut_pid;
#endif

    strncpy(s->ut_line, m->ut_line, UT_LINESIZE);

#ifdef _HAVE_UT_ID
    strncpy(s->ut_id, m->ut_id, 4);
#endif

    strncpy(s->ut_user, m->ut_user, UT_NAMESIZE);

#ifdef _HAVE_UT_HOST
    strncpy(s->ut_host, m->ut_host, UT_HOSTSIZE);
#endif

#ifdef _HAVE_UT_EXIT
    s->ut_exit.e_termination = m->e_termination;
    s->ut_exit.e_exit = m->e_exit;
#endif

#ifdef _HAVE_UT_TV
    s->ut_tv.tv_sec = m->tv_sec;
    s->ut_tv.tv_usec = m->tv_usec;
#else
    s->ut_time = m->tv_sec;
#endif

#ifdef _HAVE_UT_ADDR_V6
    memcpy(s->ut_addr_v6, m->ut_addr_v6, 4 * sizeof(int32_t));
#endif

#ifdef _HAVE_UT_SESSION
    s->ut_session = m->ut_session;
#endif
}

/* #define OUTVALUE(entry) \
                "(iissssiiiiii)", \
                entry->ut_type, \
                entry->ut_pid, \
                entry->ut_line, \
                entry->ut_id, \
                entry->ut_user, \
                entry->ut_host, \
                entry->ut_exit.e_termination, \
                entry->ut_exit.e_exit, \
                entry->ut_session, \
                entry->ut_tv.tv_sec, \
                entry->ut_tv.tv_usec, \
                entry->ut_addr_v6
*/

#define OUTVALUE(entry) \
                "(iissssiiiiii)", \
                entry->ut_type, \
                entry->ut_pid, \
                entry->ut_line, \
                entry->ut_id, \
                entry->ut_user, \
                entry->ut_host, \
                entry->e_termination, \
                entry->e_exit, \
                entry->ut_session, \
                entry->tv_sec, \
                entry->tv_usec, \
                entry->ut_addr_v6


#ifndef _HAVE_SETUTENT

static FILE *_utmpfile = NULL;
static struct utmp _utmprec;

/* open file rw, if that fails, open it read-only */
/* this is not perfect, of course... to be redone later*/
FILE *safe_open(name)
char *name;
{
    FILE *f;
/*    f = fopen(name, "rb+");
    if (!f)
	f = fopen(name, "rb");
*/

    f = fopen(name, "rb");
    return f;
}


unsigned short int checkfile(void)
{
    if (!_utmpfile) {
	_utmpfile = safe_open(UTMP_FILE);
	if (!_utmpfile) {
	    PyErr_SetFromErrno(PyExc_IOError);
	    return 0;
	}
    }
    return 1;
}
#endif

#ifndef _HAVE_UTMPNAME
void utmpname(name)
char *name;
{
    _utmpfile = safe_open(name);
	if (!_utmpfile) {
	    PyErr_SetFromErrno(PyExc_IOError);
}
}
#endif


static PyObject *utmp_utmpname(self, args)
PyObject *self;
PyObject *args;
{
    char *fname;

    if (!PyArg_ParseTuple(args, "s", &fname))
	return NULL;
    utmpname(fname);
    Py_INCREF(Py_None);
    return Py_None;
}


#ifndef _HAVE_SETUTENT
void setutent(void)
{
    int r;
    r = checkfile();
    if (!r)
	return;
    rewind(_utmpfile);
}
#endif


static PyObject *utmp_setutent(self, args)
PyObject *self;
PyObject *args;
{
    if (!PyArg_ParseTuple(args, ""))
	return NULL;
    setutent();
    Py_INCREF(Py_None);
    return Py_None;
}


#ifndef _HAVE_GETUTENT
struct utmp *getutent()
{
    int l;
    unsigned short int r;
    r = checkfile();
    if (!r)
	return NULL;
    l = fread(&_utmprec, sizeof(_utmprec), 1, _utmpfile);
    if (l == 1)
	return &_utmprec;
    else
	return NULL;
}
#endif


static PyObject *utmp_getutent(self, args)
PyObject *self;
PyObject *args;
{
    struct utmp *entry;
    struct myutmp myentry;

    if (!PyArg_ParseTuple(args, ""))
	return NULL;
    entry = getutent();

    if (!entry) {
	Py_INCREF(Py_None);
	return Py_None;
    };
    system2my(&myentry, entry);

    return Py_BuildValue(OUTVALUE((&myentry)));
}


#ifndef _HAVE_GETUTID
struct utmp *getutid(u)
struct utmp *u;
{
    return NULL;
}
#endif


static PyObject *utmp_getutid(self, args) /* args are ut_type=...[, ut_id=...] */
PyObject *self;
PyObject *args;
{
    struct myutmp myentry;
    struct utmp entry;
    struct utmp *nentry;
    struct myutmp mynentry;
    short ut_type;
    char *ut_id = "";


    if (!PyArg_ParseTuple(args, "h|s", &ut_type, &ut_id))
	return NULL;

    myentry.ut_type = ut_type;
    strncpy(myentry.ut_id, ut_id, 4);
    my2system(&entry, &myentry);
    nentry = getutid(&entry);

    if (!nentry) {
	Py_INCREF(Py_None);
	return Py_None;
    };

    system2my(&mynentry, nentry);
    return Py_BuildValue(OUTVALUE((&mynentry)));
}


#ifndef _HAVE_GETUTLINE
struct utmp *getutline(u)
struct utmp *u;
{
    struct utmp *a;
    while ( (a = getutent()) ) {
	if (!strncmp(a->ut_line, u->ut_line, UT_LINESIZE))
	    return a;
    }
    return NULL;
}
#endif


static PyObject *utmp_getutline(self, args) /* args is ut_line */
PyObject *self;
PyObject *args;
{
    struct utmp entry;
    struct utmp *nentry;
    struct myutmp mynentry;
    char *ut_line;


    if (!PyArg_ParseTuple(args, "s", &ut_line))
	return NULL;

    strncpy(entry.ut_line, ut_line, UT_LINESIZE);
    nentry = getutline(&entry);

    if (!nentry) {
	Py_INCREF(Py_None);
	return Py_None;
    };

    system2my(&mynentry, nentry);
    return Py_BuildValue(OUTVALUE((&mynentry)));
}

#ifndef _HAVE_PUTUTLINE
void pututline()
{
}
#endif


static PyObject *utmp_pututline(self, args)
PyObject *self;
PyObject *args;
{
    struct utmp entry;
    struct myutmp myentry;
    char *ut_line, *ut_id, *ut_user, *ut_host;

    if (!PyArg_ParseTuple(args, "iissssiiiiii",
			  &myentry.ut_type, &myentry.ut_pid, &ut_line,
			  &ut_id, &ut_user, &ut_host,
			  &myentry.e_termination, &myentry.e_exit,
			  &myentry.ut_session, &myentry.tv_sec,
			  &myentry.tv_usec, &myentry.ut_addr_v6))
	return NULL;

    strncpy(myentry.ut_line, ut_line, UT_LINESIZE);
    strncpy(myentry.ut_id, ut_id, 4);
    strncpy(myentry.ut_user, ut_user, UT_NAMESIZE);
    strncpy(myentry.ut_host, ut_host, UT_HOSTSIZE);

    my2system(&entry, &myentry);
#ifdef _HAVE_UT_TYPE
    pututline(&entry);
#else
    if (myentry.ut_type != USER_PROCESS) {
	memset(&entry.ut_user, 0, UT_NAMESIZE);
	memset(&entry.ut_host, 0, UT_HOSTSIZE);
        }
    pututline(&entry);
#endif
    Py_INCREF(Py_None);
    return Py_None;
}


#ifndef _HAVE_ENDUTENT
void endutent()
{
    if (_utmpfile) {
	fclose(_utmpfile);
	_utmpfile = NULL;
    }
}
#endif


static PyObject *utmp_endutent(self, args)
PyObject *self;
PyObject *args;
{
    if (!PyArg_ParseTuple(args, ""))
	return NULL;
    endutent();
    Py_INCREF(Py_None);
    return Py_None;
}


static PyMethodDef UtmpMethods[] = {
    {"utmpname", utmp_utmpname, METH_VARARGS},
    {"setutent", utmp_setutent, METH_VARARGS},
    {"getutent", utmp_getutent, METH_VARARGS},
    {"endutent", utmp_endutent, METH_VARARGS},
    {"getutid", utmp_getutid, METH_VARARGS},
    {"getutline", utmp_getutline, METH_VARARGS},
    {"pututline", utmp_pututline, METH_VARARGS},
    {NULL, NULL}		/* Sentinel */
};


void initutmpaccess()
{
    (void) Py_InitModule("utmpaccess", UtmpMethods);
}
